import os
import unittest, re
import util

class TestCase(unittest.TestCase):
	def vfailIf(self, expr, msg=None):
		"Include the expr in the error message"
		if expr:
			if msg:
				msg = '%s (%r)'%(msg, expr)
			else:
				msg = '(%r)'%(expr,)
			raise self.failureException, msg

	def vfailUnlessEqual(self, first, second, msg=None):
		"Include the exprs in the error message even if msg is given"
		if first != second:
			if msg:
				msg = '(%r != %r) %s'%(first, second, msg)
			else:
				msg = '(%r != %r)'%(first, second)
			raise self.failureException, msg

	def vfailUnlessListEqual(self, first, second, msg=None):
		self.vfailIf(util.listcmp(first, second), msg)


class CommonTests:
	def setUp(self):
		self.reset_tarballer()
	
	def tearDown(self):
		del self.tarballer

	def reset_tarballer(self):
		self.tarballer = util.Tarballer('foo')

	def do_import(self, *args, **kw):
		if kw.has_key('vers'):
			tarargs = [self.tarballer.tarpath(ver) for ver in kw['vers']]
		else:
			tarargs = [self.tarballer.tardir]
		util.safesystem(self.repos.importer, *args + ('foo', ) + tuple(tarargs))

	def do_addver(self, *args, **kw):
		self.tarballer.addver(*args, **kw)
		
	def do_verify(self, ver):
		self.repos.checkout('foo',ver)
		self.vfailIf(self.tarballer.verify(ver, self.repos.codir), "testing version %s:"%ver)
	
	def do_verify_all(self):
		for ver in self.tarballer.vflist.keys():
			self.do_verify(ver)

		self.repos.checkout('foo')
		self.vfailIf(self.tarballer.verify(self.tarballer.curver, self.repos.codir), "testing head version:")

	def test_simple(self):
		self.do_addver("1.0", add=["file1","dir1/file2"])
		self.do_import()
		self.do_verify_all()

	def test_binary(self):
		allchars = ''.join(map(chr, range(0,256)))
		self.do_addver("1.0", add=['tfile1', 'forced.bin','dir1/tfile2','dir1/force2.bin'], addextra={'bfile1':allchars, 'forced.txt':allchars, 'dir1/bfile2':allchars, 'dir1/force2.txt':allchars,})
		self.do_import('-t','*txt','-b','*.bin')
		self.do_verify_all()
		self.failIf(self.repos.is_binary("tfile1"))
		self.failIf(self.repos.is_binary("forced.txt"))
		self.failUnless(self.repos.is_binary("bfile1"))
		self.failUnless(self.repos.is_binary("forced.bin"))
		self.failIf(self.repos.is_binary(os.path.join("dir1","tfile2")))
		self.failIf(self.repos.is_binary(os.path.join("dir1","force2.txt")))
		self.failUnless(self.repos.is_binary(os.path.join("dir1","bfile2")))
		self.failUnless(self.repos.is_binary(os.path.join("dir1","force2.bin")))
	
	def test_add(self):
		self.do_addver("1.0", add=["file1","dir1/filea"])
		self.do_addver("1.1", add=["file2","dir1/fileb","dir2/filec"])
		self.do_import()
		self.do_verify_all()

	def test_rm(self):
		self.do_addver("1.0", add=["file1","file2","dir1/filea","dir1/fileb","dir2/filec"])
		self.do_addver("1.1", rm=["file1","dir1/filea","dir2/filec"])
		self.do_import()
		self.do_verify_all()

	def test_exclude(self):
		self.do_addver("1.0", add=["1.foo.bar","2.foo","dir1/a.foo.bar","dir1/b.foo", "dir.foo/f1"])
		self.do_addver("1.1", add=["3.foo.bar","4.foo","dir2/c.foo.bar","dir2/d.foo", "dir.foo/f2"])
		self.do_import('-x', '*.foo')
		self.reset_tarballer()
		self.do_addver("1.0", add=["1.foo.bar","dir1/a.foo.bar"])
		self.do_addver("1.1", add=["3.foo.bar","dir2/c.foo.bar"])
		self.do_verify_all()

	def test_generated(self):
		self.do_addver("1.0", add=["foo.in","foo","dir1/bar.in", "dir1/bar", "dir2/baz.in", "dir2/Makefile.am", "dir2/Makefile.in", "dir2/Makefile"])
		self.do_import()
		self.reset_tarballer()
		self.do_addver("1.0", add=["foo.in","dir1/bar.in", "dir2/baz.in", "dir2/Makefile.am"])
		self.do_verify_all()
		self.vfailUnlessEqual(self.repos.get_ignore_list(os.curdir), ["foo"])
		self.vfailUnlessEqual(self.repos.get_ignore_list("dir1"), ["bar"])
		self.vfailUnlessListEqual(self.repos.get_ignore_list("dir2"), ["baz", "Makefile.in", "Makefile"])

	def test_generated_autoconf(self):
		self.do_addver("1.0", add=["dir1/configure.in", "dir1/config.log", "dir1/config.status", "dir1/config.cache", "dir1/configure"])
		self.do_import()
		self.reset_tarballer()
		self.do_addver("1.0", add=["dir1/configure.in"])
		self.do_verify_all()
		self.vfailUnlessListEqual(self.repos.get_ignore_list("dir1"), ["config.log", "config.status", "config.cache", "configure"])

	def test_generated_autoconf2(self):
		self.do_addver("1.0", add=["dir1/configure.ac", "dir1/config.log", "dir1/config.status", "dir1/config.cache", "dir1/configure"])
		self.do_import()
		self.reset_tarballer()
		self.do_addver("1.0", add=["dir1/configure.ac"])
		self.do_verify_all()
		self.vfailUnlessListEqual(self.repos.get_ignore_list("dir1"), ["config.log", "config.status", "config.cache", "configure"])

	def test_generated_distutils(self):
		self.do_addver("1.0", add=["dir1/setup.py", "dir1/setup.pyc", "dir1/PKG-INFO", "dir1/build/foo", "dir1/dist/foo"])
		self.do_import()
		self.reset_tarballer()
		self.do_addver("1.0", add=["dir1/setup.py"])
		self.do_verify_all()
		self.vfailUnlessListEqual(self.repos.get_ignore_list("dir1"), ["*.py[co]", "PKG-INFO", "build", "dist"])

	def test_generated_no_ignore(self):
		self.do_addver("1.0", add=["foo.in", "foo.py", "foo.pyc", "foo", "dir1/bar.in", "dir1/bar", "dir1/bar.py", "dir1/bar.pyo", "dir2/baz.in"])
		self.do_import('--no-cvsignore')
		self.reset_tarballer()
		self.do_addver("1.0", add=["foo.in", "foo.py", "dir1/bar.in", "dir1/bar.py", "dir2/baz.in"])
		self.do_verify_all()
		self.vfailUnlessEqual(self.repos.get_ignore_list(os.curdir), [])
		self.vfailUnlessEqual(self.repos.get_ignore_list("dir1"), [])
		self.vfailUnlessEqual(self.repos.get_ignore_list("dir2"), [])

	def test_keep_generated(self):
		self.do_addver("1.0", add=["foo.in","foo"])
		self.do_import('--keep-generated')
		self.do_verify_all()

	def test_generated_pat(self):
		self.do_addver("1.0", add=["foo.py","foo.pyc","dir1/bar.py","dir1/bar.pyc", "dir2/baz"])
		self.do_import()
		self.reset_tarballer()
		self.do_addver("1.0", add=["foo.py","dir1/bar.py", "dir2/baz"])
		self.do_verify_all()
		self.vfailUnlessEqual(self.repos.get_ignore_list(os.curdir), ["*.py[co]"])
		self.vfailUnlessEqual(self.repos.get_ignore_list("dir1"), ["*.py[co]"])
		self.vfailUnlessEqual(self.repos.get_ignore_list("dir2"), [])

	def test_cmdline_sort(self):
		self.do_addver("1.0", add=["file1"])
		self.do_addver("1.1-pre1", add=["file2"])
		self.do_addver("1.1-pre3", add=["file3"])
		self.do_addver("1.1-pre19", add=["file4"])
		self.do_addver("1.1-rc1", add=["file10"])
		self.do_addver("1.1-rc2", add=["file8"])
		self.do_addver("1.1-rc19", add=["file9"])
		self.do_addver("1.1", add=["file5"])
		self.do_addver("1.2", add=["file6"])
		self.do_addver("1.19", add=["file7"])
		self.do_import(vers=["1.19","1.1-rc19","1.1-rc2","1.1","1.0","1.1-pre3","1.1-pre1","1.2","1.1-rc1","1.1-pre19"])
		self.do_verify_all()
		log = self.repos.get_log("file1")
		lastpos = -1
		for ver in "1.19", "1.2", "1.1", "1.1-rc19", "1.1-rc2", "1.1-rc1", "1.1-pre19", "1.1-pre3", "1.1-pre1", "1.0":
			x = re.search('^(.*)foo '+re.escape(ver)+r'\s', log, re.DOTALL)
			self.failUnless(x)
			vpos = len(x.group(0))
			print 'ver',ver,'pos',vpos
			self.failIf(vpos<=lastpos)
			lastpos = vpos

	def test_cmdline_no_sort(self):
		self.do_addver("1.1", add=["file1"])
		self.do_addver("1.0", add=["file2"])
		self.do_import("--no-sort", vers=["1.1","1.0"])
		self.do_verify_all() #this is sufficient since the head test will make sure 1.0 was the last checked in.

